package cn.newfeifan.mall.module.distri.service.integral;

import cn.newfeifan.mall.module.distri.controller.admin.ptprofitlog.vo.PtProfitLogSaveReqVO;
import cn.newfeifan.mall.module.distri.dal.dataobject.integral.IntegralDO;
import cn.newfeifan.mall.module.distri.dal.dataobject.orderpercentage.OrderPercentageDO;
import cn.newfeifan.mall.module.distri.dal.dataobject.ptprofitlog.PtProfitLogDO;
import cn.newfeifan.mall.module.distri.dal.mysql.integral.IntegralMapper;
import cn.newfeifan.mall.module.distri.dal.mysql.ptprofit.PtProfitMapper;
import cn.newfeifan.mall.module.distri.enums.CaclEnum;
import cn.newfeifan.mall.module.distri.service.orderpercentage.OrderPercentageService;
import cn.newfeifan.mall.module.distri.service.ptprofitlog.PtProfitLogService;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Map;

/**
 * 推荐用户积分 Service 实现类
 *
 * @author 非繁人
 */
@Service
@Validated
public class IntegralServiceImpl implements IntegralService {

    @Resource
    private PtProfitMapper ptProfitMapper;
    @Resource
    private IntegralMapper integralMapper;
    @Resource
    private PtProfitLogService ptProfitLogService;
    @Resource
    private OrderPercentageService orderPercentageService;

    /**
     * add by Ben
     * 修改用户积分
     *
     * @param userId       要修改积分的用户
     * @param caclEnum     导致积分变化的事件类型枚举
     * @param amount       用户可用积分变化金额，可为负数
     * @param freezeAmount 用户冻结积分变化金额，可为负数
     */
    @Override
    public void updateUserIntegral(Long generateUserId,//导致产生积分变动的用户ID
                                   Long userId, CaclEnum caclEnum, Long amount, Long freezeAmount,
                                   Long tradeOrderId,
                                   String orderNum
    ) {
        //修改用户积分钱包的积分
        IntegralDO integralDO = integralMapper.selectOne(IntegralDO::getUserId, userId);

        // 退款返回 为 +
        integralDO.setCurrentQuota(integralDO.getCurrentQuota() + amount);

        // 直推奖超出额度
        Long ancestorQuotaAmount = 0L;
        // 如果是推荐人
        if (caclEnum.getType().equals(CaclEnum.ORDER_REFUND_INTEGRAL_ANCESTER.getType())) {

            PtProfitLogDO profitLog = ptProfitLogService.selectProfitLogByUserIdWithProfitStatus(userId, CaclEnum.RECOMMENDED_PERSON_QUOTA.getType(), orderNum);

            if(profitLog.getAncestorQuotaAmount() != null && profitLog.getAncestorQuotaAmount() != 0){
                ancestorQuotaAmount = profitLog.getAncestorQuotaAmount();
            }
            integralDO.setCurrentQuota(integralDO.getCurrentQuota() + ancestorQuotaAmount);
            // 退回扣减的最大可用额度,和累加获得的直推奖额度
            integralDO.setHighQuota(integralDO.getHighQuota() - amount - ancestorQuotaAmount);
            integralDO.setAncestorQuota(integralDO.getAncestorQuota() + amount + ancestorQuotaAmount);
        }

        integralMapper.updateById(integralDO);

        //用户修改后的当前可用的钱包余额
        Long afterAmount = null;
        //用户修改后的冻结积分钱包余额
        Long afterFreezeAmount = null;

        //查询用户修改后的积分
        Map<String, Object> map = ptProfitMapper.selectUserIntegral(userId);
        Object currentQuotaObj = map.get("current_quota");
        Object freezeQuotaObj = map.get("freeze_quota");

        afterAmount = (Long) currentQuotaObj;
        afterFreezeAmount = (Long) freezeQuotaObj;

        //记录用户的积分变动日志
        if (caclEnum.getType().equals(CaclEnum.ORDER_REFUND_INTEGRAL_ANCESTER.getType())) {
            OrderPercentageDO orderPercentageDO = orderPercentageService.queryStatus();
            ptProfitLogService.addMessage(generateUserId, userId, caclEnum, amount + ancestorQuotaAmount, afterAmount,
                    freezeAmount, afterFreezeAmount, null, tradeOrderId, orderNum, -amount - ancestorQuotaAmount, integralDO.getHighQuota() + Long.parseLong(orderPercentageDO.getBaseMaxQuota()));
        } else {
            ptProfitLogService.addMessage(generateUserId, userId, caclEnum, amount, afterAmount,
                    freezeAmount, afterFreezeAmount, null, tradeOrderId, orderNum, null, null);
        }

    }

    @Override
    public void updateIntegralFreezeHighQuota(Long generateUserId, Long userId, CaclEnum caclEnum, Long totalFreezeHighQuota, Long tradeOrderId, String orderNum) {
        IntegralDO integralDO = integralMapper.selectOne(IntegralDO::getUserId, userId);
//        integralDO.setFreezeHighQuota(integralDO.getFreezeHighQuota() - totalFreezeHighQuota);
        integralDO.setHighQuota(integralDO.getHighQuota() - totalFreezeHighQuota);
        integralDO.setHighQuotaTotal(integralDO.getHighQuotaTotal() - totalFreezeHighQuota);
        integralMapper.updateById(integralDO);

        OrderPercentageDO orderPercentageDO = orderPercentageService.queryStatus();

        //记录用户的积分变动日志
        PtProfitLogSaveReqVO profitLogSaveReqVO = PtProfitLogSaveReqVO.builder()
//                .freezeHighQuota(-totalFreezeHighQuota)
//                .afterFreezeHighQuota(integralDO.getFreezeHighQuota())
                .maxAvailablePointsAmount(-totalFreezeHighQuota)
                .afterMaxAvailablePointsAmount(integralDO.getHighQuota() + Long.parseLong(orderPercentageDO.getBaseMaxQuota()))     // 系统设置的最大可用积分
                .userId(userId)
                .generateUserId(generateUserId)
                .profitStatus(caclEnum.getType())
                .orderId(tradeOrderId)
                .orderNo(orderNum)
                .build();
        ptProfitLogService.createPtProfitLog(profitLogSaveReqVO);
    }

//
//    /**
//     * 修改用户身价
//     * @param userId 用户ID
//     * @param socialStatusUpdateEnum 身价变动类别
//     * @param socialStatusPoint 身价变化值
//     * @param ObjectId 导致身价变动的对应业务对象ID，如订单ID、收藏记录ID、新注册用户ID等
//     * @return 升级后的身价名称，如果返回null，表明没升级
//     */
//    public String updateUserSocialStatus(Long userId, SocialStatusUpdateEnum socialStatusUpdateEnum, Integer socialStatusPoint,
//                                   Long ObjectId){
//        //升级后的身价名称，如果返回null，表明没升级
//        String newSocialStatusName = null;
//        //用户身价升了几级
//        Integer socialStatusUpgradation = 0;
//
//        DuserDO duser = duserMapper.selectOne("user_id", userId);
//        Integer oldSocialStatus = duser.getSocialStatus();//升级前身价等级
//
//        return newSocialStatusName;
//    }
}